@@ -1,13 +1,17 @@
 var topics =
 [
     [ "Basic Allocation", "group__malloc.html", "group__malloc" ],
-    [ "Extended Functions", "group__extended.html", "group__extended" ],
     [ "Aligned Allocation", "group__aligned.html", "group__aligned" ],
-    [ "Heap Allocation", "group__heap.html", "group__heap" ],
-    [ "Zero initialized re-allocation", "group__zeroinit.html", "group__zeroinit" ],
     [ "Typed Macros", "group__typed.html", "group__typed" ],
+    [ "Zero initialized re-allocation", "group__zeroinit.html", "group__zeroinit" ],
+    [ "Heaps", "group__heap.html", "group__heap" ],
     [ "Heap Introspection", "group__analysis.html", "group__analysis" ],
+    [ "Arenas", "group__arenas.html", "group__arenas" ],
+    [ "Subprocesses", "group__subproc.html", "group__subproc" ],
+    [ "Extended Functions", "group__extended.html", "group__extended" ],
+    [ "Statistics", "group__stats.html", "group__stats" ],
     [ "Runtime Options", "group__options.html", "group__options" ],
+    [ "Thread-local heaps", "group__theap.html", "group__theap" ],
     [ "Posix", "group__posix.html", "group__posix" ],
     [ "C++ wrappers", "group__cpp.html", "group__cpp" ]
 ];
\ No newline at end of file
